"""
Enumeration: ECapability

This enumeration defines different capabilities.

Enumerations:
    ECapability: Represents various capabilities.
"""

from enum import Enum


class ECapability(Enum):
    """
    Represents various capabilities.

    Enumerations:
        POWER: Power capability.
        RSSI: RSSI (Received Signal Strength Indication) capability.
        OTA: OTA (Over-the-Air) capability.
        DETECT: Detection capability.
        BATTERY: Battery capability.
        TOGGLE: Toggle capability.
        PERCENTAGE: Percentage capability.
        MOTOR_CONTROL: Motor control capability.
        MOTOR_REVERSE: Motor reverse capability.
        MOTOR_CLB: Motor calibration capability.
        TEMPERATURE: Temperature capability.
        HUMIDITY: Humidity capability.
        PRESS: Pressure capability.
        COLOR_RGB: RGB color capability.
        COLOR_TEMPERATURE: Color temperature capability.
        BRIGHTNESS: Brightness capability.
        CAMERA_STREAM: Camera stream capability.
        STARTUP: Startup capability.
    """
    POWER = "power"
    RSSI = "rssi"
    OTA = "ota"
    DETECT = "detect"
    BATTERY = "battery"
    TOGGLE = "toggle"
    PERCENTAGE = "percentage"
    MOTOR_CONTROL = "motor-control"
    MOTOR_REVERSE = "motor-reverse"
    MOTOR_CLB = "motor-clb"
    TEMPERATURE = "temperature"
    HUMIDITY = "humidity"
    PRESS = "press"
    COLOR_RGB = "color-rgb"
    COLOR_TEMPERATURE = "color-temperature"
    BRIGHTNESS = "brightness"
    CAMERA_STREAM = "camera-stream"
    STARTUP = "startup"



"""
Module: baseClassDevice

This module provides the BaseClassDevice API.

Classes:
    BaseClassDevice: Represents the BaseClassDevice API.

Usage:
    from .baseClassDevice import BaseClassDevice
"""

from typing import Dict, Any

from ..ts.enum.EMethod import EMethod
from ..ts.enum.EPath import EPath
from ..utils.httpUtils import httpUtils


class BaseClassDevice(httpUtils):
    """
    Represents the BaseClassDevice API.

    Methods:
        discoverySubDevices(params: Dict[str, Any]) -> Dict[str, Any]: 
            Searches for sub-devices.
        manualAddSubDevice(params: Dict[str, Any]) -> Dict[str, Any]: 
            Manually adds a sub-device.
        getDeviceList() -> Dict[str, Any]: 
            Gets the device list.
        updateDeviceState(serial_number: str, updateParams: Dict[str, Any]) -> Dict[str, Any]: 
            Updates the status of a specified device.
        deleteDevice(serial_number: str) -> Dict[str, Any]: 
            Deletes a device.
    """

    async def discoverySubDevices(self, params: Dict[str, Any]) -> Dict[str, Any]:
        """
        Searches for sub-devices.

        Args:
            params (Dict[str, Any]): Parameters for the request.

        Returns:
            Dict[str, Any]: Dictionary containing the response data.
        """
        return await self.httpRequest(
            path=EPath.DEVICE_DISCOVERY.value,
            method=EMethod.PUT,
            params=params
        )

    async def manualAddSubDevice(self, params: Dict[str, Any]) -> Dict[str, Any]:
        """
        Manually adds a sub-device (currently only supports adding RTSP cameras and ESP32 cameras).

        Args:
            params (Dict[str, Any]): Parameters for the request.

        Returns:
            Dict[str, Any]: Dictionary containing the response data.
        """
        return await self.httpRequest(
            path=EPath.DEVICE.value,
            method=EMethod.POST,
            params=params
        )

    async def getDeviceList(self) -> Dict[str, Any]:
        """
        Gets the device list.

        Returns:
            Dict[str, Any]: Dictionary containing the response data.
        """
        return await self.httpRequest(
            path=EPath.DEVICE.value,
            method=EMethod.GET,
        )

    async def updateDeviceState(self, serial_number: str, updateParams: Dict[str, Any]) -> Dict[str, Any]:
        """
        Updates the information or status of a specified device.

        Args:
            serial_number (str): Serial number of the device.
            updateParams (Dict[str, Any]): Parameters for updating the device.

        Returns:
            Dict[str, Any]: Dictionary containing the response data.
        """
        return await self.httpRequest(
            path=f"{EPath.DEVICE.value}/{serial_number}",
            method=EMethod.PUT,
            params=updateParams
        )

    async def deleteDevice(self, serial_number: str) -> Dict[str, Any]:
        """
        Deletes a device.

        Args:
            serial_number (str): Serial number of the device to be deleted.

        Returns:
            Dict[str, Any]: Dictionary containing the response data.
        """
        return await self.httpRequest(
            path=f"{EPath.DEVICE.value}/{serial_number}",
            method=EMethod.DELETE
        )



"""
Enumeration: EMethod

This enumeration defines different HTTP methods.

Enumerations:
    EMethod: Represents various HTTP methods.
"""

from enum import Enum


class EMethod(Enum):
    """
    Represents various HTTP methods.

    Enumerations:
        GET: HTTP GET method.
        PUT: HTTP PUT method.
        POST: HTTP POST method.
        DELETE: HTTP DELETE method.
    """
    GET = "get"
    PUT = "put"
    POST = "post"
    DELETE = "delete"



"""
Module: ewelink_cube

This module provides classes for interacting with eWeLink Cube APIs.

Classes:
    EWelinkCube: The main class for creating eWeLink Cube APIs.
"""
#pylint: disable-msg=too-few-public-methods

import logging
import os

from .api.ihostClass import IHostClass
from .api.nspanelproClass import NSPanelProClass

# Set logger level
if 'LOG_LEVEL' in os.environ:
    log_level = os.environ['LOG_LEVEL']
else:
    log_level = logging.INFO
logging.basicConfig(level=log_level)


class EWelinkCube:
    """
    The main class for creating eWeLink Cube APIs.
    """

    def create_api(self, api_name, **args):
        """
        Create an instance of the specified API.

        Args:
            api_name (str): The name of the API.
            **args: Additional arguments to be passed to the API constructor.

        Returns:
            object: An instance of the specified API.

        Raises:
            ValueError: If the API name is invalid.
        """
        if api_name == 'iHost':
            return IHostClass(**args)

        if api_name == 'nsPanelPro':
            return NSPanelProClass(**args)

        raise ValueError("Invalid API name")



"""
Module: httpUtils

This module provides utility functions for making HTTP requests.

Classes:
    httpUtils: Contains functions for making HTTP requests.
"""
#pylint: disable-msg=too-few-public-methods

from typing import Dict, Any, Optional

import logging
import json
import aiohttp

from ..ts.enum.EMethod import EMethod
from ..ts.enum.EPath import EPath
from ..ts.interface.IConfig import IConfig
from ..ts.interface.IResponse import IResponse

_LOGGER = logging.getLogger(__name__)


class httpUtils(IConfig):
    """
    Contains functions for making HTTP requests.
    """
    session = None

    async def httpRequest(
        self,
        path: str,
        method: EMethod,
        params: Optional[Dict[str, Any]] = None,
        isNeedAT: bool = True
    ) -> IResponse:
        """
        Makes an HTTP request.

        Args:
            path (str): API path.
            method (EMethod): HTTP method.
            params (Optional[Dict[str, Any]]): Request parameters (default: None).
            isNeedAT (bool): Flag to indicate if access token is required (default: True).

        Returns:
            IResponse: Dictionary containing the response data.
        """
        url = f"http://{self.ip}{EPath.ROOT.value}{EPath.V1.value}{path}"
        _LOGGER.debug(f'httpRequest {method}: {url}')

        headers = {'Content-Type': 'application/json'}
        if isNeedAT and self.at:
            headers['Authorization'] = f'Bearer {self.at}'
        _LOGGER.debug(f'httpRequest headers: {headers}')

        response: IResponse = None
        async with aiohttp.ClientSession(headers=headers) as session:
            if method == EMethod.GET:
                async with session.get(url, params=params) as resp:
                    response = await resp.text()
            elif method == EMethod.POST:
                params = {} if not params else params
                async with session.post(url, data=json.dumps(params)) as resp:
                    response = await resp.text()
            elif method == EMethod.PUT:
                async with session.put(url, data=json.dumps(params)) as resp:
                    response = await resp.text()
            elif method == EMethod.DELETE:
                async with session.delete(url, params=params) as resp:
                    response = await resp.text()

        try:
            _LOGGER.debug(f'httpRequest response: {response}')
            if response is None:
                return {'error': -1, 'msg': 'Empty response', 'data': {}}
            return json.loads(response)
        except json.JSONDecodeError:
            _LOGGER.error(f'httpRequest response: {response}')
            return {'error': -1, 'msg': 'Invalid JSON response', 'data': {}}



"""
Module: BaseClassSse

This module provides a base class for handling Server-Sent Events (SSE) connections.

Classes:
    BaseClassSse: Represents a base class for SSE connections.
"""
#pylint: disable-msg=broad-exception-caught

from typing import Dict, Any, Optional, Callable

import logging
import json
import asyncio
import aiohttp

from ..config import Store
from ..ts.enum.EPath import EPath
from ..ts.interface.ISseEvent import ISseEvent

_LOGGER = logging.getLogger(__name__)


class BaseClassSse(Store):
    """
    Represents a base class for SSE connections.
    """

    event_listeners = None
    session = None

    async def init_sse(self, session: aiohttp.ClientSession = None):
        """
        Initialize SSE connection.

        Returns:
            Optional dictionary containing the response data.
        """
        # Check for necessary parameters
        if not self.ip:
            error_msg = {"error": 1000, "msg": "ip is needed", "data": {}}
            _LOGGER.error(error_msg)
            return error_msg
        if not self.at:
            error_msg = {"error": 1000, "msg": "at is needed", "data": {}}
            _LOGGER.error(error_msg)
            return error_msg

        _LOGGER.debug('Initialize SSE session.')

        # Set session
        self.session = session if session is not None else aiohttp.ClientSession()

        # Set SSE event listeners
        self.event_listeners = {}

        # Create SSE connection URL
        url = f"http://{self.ip}{EPath.ROOT.value}{EPath.SSE.value}?access_token={self.at}"
        _LOGGER.debug(f'SSE Init: {url}')

        try:
            # Start SSE connection in a separate task
            asyncio.create_task(self.handle_sse(url))
            return True
        except Exception as error:
            # Handle exception in case of error
            _LOGGER.error(f'SSE Error: {url}')
            _LOGGER.error(f'SSE Error: {error}')

        return False

    async def handle_sse(self, url: str) -> None:
        """
        Handle SSE connection.

        Args:
            url (str): The SSE connection URL.
        """
        while True:
            try:
                if 'onopen' in self.event_listeners:
                    await self.event_listeners["onopen"]()
                else:
                    _LOGGER.debug('Connected to SSE server.')

                async with self.session.get(url) as response:
                    event_name = None
                    async for line in response.content:
                        event = line.decode().strip()
                        if event.startswith("event:"):
                            event_name = event[6:].strip()
                        elif event.startswith("data:") and event_name:
                            event_data = event[5:]
                            if event_data.strip() != "":
                                await self.handle_event(event_name, event_data)
                                event_name = None

            except asyncio.TimeoutError:
                if 'onerror' in self.event_listeners:
                    _LOGGER.debug('SSE time-out error, reconnecting.')
                    await self.event_listeners["onerror"](asyncio.TimeoutError)
                else:
                    _LOGGER.warning('SSE time-out error, reconnecting.')
                await self.close()
                await self.init_sse()

            except Exception as e:
                if 'onerror' in self.event_listeners:
                    await self.event_listeners["onerror"](e)
                else:
                    _LOGGER.error(f'SSE connection error: {e}')

    async def handle_event(self, event_name: str, event_data: any) -> None:
        """
        Handle SSE event.

        Args:
            event_name (str): The name of the SSE event.
            event_data (any): The data associated with the SSE event.
        """
        try:
            data = json.loads(event_data)
            _LOGGER.debug(f'Event detected: {event_name}, Data: {data}')
            if event_name and event_name in self.event_listeners:
                _LOGGER.debug(f'Event callback: {event_name}, Data: {data}')
                handler = self.event_listeners[event_name]
                await handler(data)

        except json.JSONDecodeError as error:
            _LOGGER.debug(f'Event error: {error}')
            if 'onerror' in self.event_listeners:
                await self.event_listeners["onerror"](error)

    def mount_sse_func(self, handler: ISseEvent) -> Optional[Dict[str, Any]]:
        """
        Mount SSE function.

        Args:
            handler: Handler for the SSE function.

        Returns:
            Optional dictionary containing the response data.
        """
        _LOGGER.debug('Mount SSE functions.')
        if self.event_listeners is None:
            return {'error': 1000, 'msg': 'must invoke initSSE first', 'data': {}}

        if hasattr(handler, 'onopen') and callable(handler.onopen):
            self.event_listeners.onOpen = handler.onopen

        if hasattr(handler, 'onerror') and callable(handler.onerror):
            self.event_listeners.onError = handler.onerror

        if hasattr(handler, 'onAddDevice') and callable(handler.onAddDevice):
            self.register_event_listener('device#v1#addDevice', handler.onAddDevice)

        if hasattr(handler, 'onUpdateDeviceState') and callable(handler.onUpdateDeviceState):
            self.register_event_listener('device#v1#updateDeviceState', handler.onUpdateDeviceState)

        if hasattr(handler, 'updateDeviceInfo') and callable(handler.updateDeviceInfo):
            self.register_event_listener('device#v1#updateDeviceInfo', handler.onUpdateDeviceInfo)

        if hasattr(handler, 'onUpdateDeviceOnline') and callable(handler.onUpdateDeviceOnline):
            self.register_event_listener('device#v1#updateDeviceOnline', handler.onUpdateDeviceOnline)

        if hasattr(handler, 'onDeleteDevice') and callable(handler.onDeleteDevice):
            self.register_event_listener('device#v1#deleteDevice', handler.onDeleteDevice)

        return None

    def unmount_sse_func(self) -> None:
        """
        Unmount SSE function.
        """
        if self.event_listeners:
            self.remove_event_listener('device#v1#addDevice')
            self.remove_event_listener('device#v1#updateDeviceState')
            self.remove_event_listener('device#v1#updateDeviceInfo')
            self.remove_event_listener('device#v1#updateDeviceOnline')
            self.remove_event_listener('device#v1#deleteDevice')
            self.close()

    def register_event_listener(self, event_type: str, handler: Callable) -> None:
        """
        Registers an event listener for the specified event type.

        Args:
            event_type (str): The type of the event.
            handler (Callable): The handler function to be called when the event occurs.
        """
        self.event_listeners[event_type] = handler

    def remove_event_listener(self, event_type: str) -> None:
        """
        Removes the event listener for the specified event type.

        Args:
            event_type (str): The type of the event.
        """
        if event_type in self.event_listeners:
            del self.event_listeners[event_type]

    async def close(self) -> None:
        """
        Closes the SSE session connection and releases any associated resources.
        """
        await self.session.close()



"""
Module: baseClassHardware

This module provides the BaseClassHardware API.

Classes:
    BaseClassHardware: Represents the BaseClassHardware API.

Usage:
    from .baseClassHardware import BaseClassHardware
"""

from typing import Dict, Any

from ..ts.enum.EMethod import EMethod
from ..ts.enum.EPath import EPath
from ..utils.httpUtils import httpUtils


class BaseClassHardware(httpUtils):
    """
    Represents the BaseClassHardware API.

    Methods:
        rebootBridge() -> Dict[str, Any]: Reboots the gateway.
        setSpeaker(volume: int) -> Dict[str, Any]: Controls the speaker.
    """

    async def rebootBridge(self) -> Dict[str, Any]:
        """
        Reboots the gateway.

        Returns:
            Dict[str, Any]: Dictionary containing the response data.
        """
        return await self.httpRequest(
            path=EPath.HARDWARE_REBOOT.value,
            method=EMethod.POST,
        )

    async def setSpeaker(self, play_type: str, sound: any, beep: any) -> Dict[str, Any]:
        """
        Controls the speaker.

        Args:
            play_type (str): Play sound or beep.
            sound (any): SoundObject.
            beep (any): BeepObject.

        Returns:
            Dict[str, Any]: Dictionary containing the response data.
        """
        return await self.httpRequest(
            path=EPath.HARDWARE_SPEAKER.value,
            method=EMethod.POST,
            params={'type': play_type, 'sound': sound, 'beep': beep}
        )



"""
Interface module: IThirdpartyDevice

This module defines the IThirdpartyDevice class.

Classes:
    IThirdpartyDevice: Represents a third-party device with its properties such as third serial number, name,
        display category, capabilities, state, manufacturer, model, firmware version, service address, and tags.
"""
#pylint: disable-msg=too-few-public-methods
#pylint: disable-msg=too-many-arguments
#pylint: disable-msg=too-many-instance-attributes

from typing import Optional, List

from ..enum.ECategory import ECategory
from .IDevice import ICapability


class IThirdpartyDevice:
    """
    Represents a third-party device.

    Attributes:
        third_serial_number (str): The third serial number of the device.
        name (str): The name of the device.
        display_category (ECategory): The display category of the device.
        capabilities (List[ICapability]): The list of capabilities associated with the device.
        state (any): The state of the device.
        manufacturer (str): The manufacturer of the device.
        model (str): The model of the device.
        firmware_version (str): The firmware version of the device.
        service_address (str): The service address of the device.
        tags (Optional[any]): The optional tags associated with the device.
    """

    def __init__(
        self,
        third_serial_number: str,
        name: str,
        display_category: ECategory,
        capabilities: List[ICapability],
        state: any,
        manufacturer: str,
        model: str,
        firmware_version: str,
        service_address: str,
        tags: Optional[any] = None,
    ):
        self.third_serial_number = third_serial_number
        self.name = name
        self.display_category = display_category
        self.capabilities = capabilities
        self.state = state
        self.manufacturer = manufacturer
        self.model = model
        self.firmware_version = firmware_version
        self.service_address = service_address
        self.tags = tags



"""
Interface module: IResponse

This module defines the IResponse class.

Classes:
    IResponse: Represents a response object with properties for error code, message, and data.
"""
#pylint: disable-msg=too-few-public-methods

class IResponse:
    """
    Represents a response object.

    Attributes:
        error (int): The error code of the response.
        msg (str): The message of the response.
        data (any): The data of the response.
    """

    def __init__(self, error: int, msg: str, data: any):
        self.error = error
        self.msg = msg
        self.data = data



"""
Enumeration: EPath

This enumeration defines different API paths.

Enumerations:
    EPath: Represents various API paths.
"""

from enum import Enum


class EPath(Enum):
    """
    Represents various API paths.

    Enumerations:
        ROOT: Root API path.
        V1: API v1 path.
        SSE: Server-Sent Events (SSE) API path for bridge.
        BRIDGE: Bridge API path.
        BRIDGE_TOKEN: Bridge access token API path.
        BRIDGE_RUNTIME: Bridge runtime API path.
        BRIDGE_CONFIG: Bridge configuration API path.
        HARDWARE_REBOOT: Hardware reboot API path.
        HARDWARE_SPEAKER: Hardware speaker API path.
        DEVICE_DISCOVERY: Device discovery API path.
        DEVICE: Device API path.
        THIRD_PARTY: Third-party event API path.
        DEBUG_LOG: Debug log API path.
    """
    ROOT = "/open-api"
    V1 = "/v1/rest"
    SSE = "/v1/sse/bridge"
    BRIDGE = "/bridge"
    BRIDGE_TOKEN = "/bridge/access_token"
    BRIDGE_RUNTIME = "/bridge/runtime"
    BRIDGE_CONFIG = "/bridge/config"
    HARDWARE_REBOOT = "/hardware/reboot"
    HARDWARE_SPEAKER = "/hardware/speaker"
    DEVICE_DISCOVERY = "/devices/discovery"
    DEVICE = "/devices"
    THIRD_PARTY = "/thirdparty/event"
    DEBUG_LOG = "/thirdparty/debug-log"



"""
Interface module: IThirdParty

This module defines the IHeader class.
This module defines the IEndpoint class.
This module defines the IThirdRequest class.

Classes:
    IHeader: Represents a header object with properties for name, message ID, and version.
    IEndpoint: Represents an endpoint object with properties for serial number and third serial number.
    IThirdRequest: Represents a third request object with an event dictionary.
"""
#pylint: disable-msg=too-few-public-methods

class IHeader:
    """
    Represents a header object.

    Attributes:
        name (str): The name of the header.
        message_id (str): The message ID of the header.
        version (str): The version of the header.
    """

    def __init__(self, name: str, message_id: str, version: str):
        self.name = name
        self.message_id = message_id
        self.version = version


class IEndpoint:
    """
    Represents an endpoint object.

    Attributes:
        serial_number (str): The serial number of the endpoint.
        third_serial_number (str): The third serial number of the endpoint.
    """

    def __init__(self, serial_number: str, third_serial_number: str):
        self.serial_number = serial_number
        self.third_serial_number = third_serial_number


class IThirdRequest:
    """
    Represents a third request object.

    Attributes:
        event (dict): The event dictionary of the third request.
    """

    def __init__(self, event: dict):
        self.event = event



"""
Module: baseClass

This module provides the BaseClass API.

Classes:
    BaseClass: Represents the BaseClass API.

Usage:
    from .baseClass import BaseClass
"""

from typing import Dict, Any

import logging

from ..ts.enum.EMethod import EMethod
from ..ts.enum.EPath import EPath

from .baseClassBridge import BaseClassBridge
from .baseClassDevice import BaseClassDevice
from .baseClassHardware import BaseClassHardware
from .baseClassSse import BaseClassSse

_LOGGER = logging.getLogger(__name__)


class BaseClass(BaseClassBridge, BaseClassDevice, BaseClassHardware, BaseClassSse):
    """
    Represents the BaseClass API.

    Inherits:
        BaseClassSse: Base class for APIs that support Server-Sent Events (SSE).

    Attributes:
        ip (str): IP address of the device.
        at (str): Access token for the gateway.
        debug (bool): Debug mode flag.

    Methods:
        __init__(ip: str, at: str = '', debug: bool = False): Initializes the BaseClass object.
        setIp(ip: str): Sets the IP address of the device.
        getIp() -> str: Gets the IP address of the device.
        setAT(at: str): Sets the access token for the gateway.
        getAt() -> str: Gets the access token for the gateway.
        sendCommandToDevice(deviceId: str, command: Dict[str, Any]) -> Dict[str, Any]: Sends a command to a device.
        getDebugLog() -> Dict[str, Any]: Gets the debug log interface.
    """

    def __init__(self, ip: str, at: str = '', debug: bool = False):
        """
        Initializes the BaseClass object.

        Parameters:
            ip (str): IP address of the device.
            at (str): Access token for the gateway.
            debug (bool): Debug mode flag.
        """
        super().__init__()
        self.ip: str = ip
        self.at: str = at
        self.debug: bool = debug

    def setIp(self, ip: str):
        """
        Sets the IP address of the device.

        Parameters:
            ip (str): IP address of the device.
        """
        self.ip = ip

    def getIp(self) -> str:
        """
        Gets the IP address of the device.

        Returns:
            str: IP address of the device.
        """
        return self.ip

    def setAT(self, at: str):
        """
        Sets the access token for the gateway.

        Parameters:
            at (str): Access token for the gateway.
        """
        self.at = at

    def getAt(self) -> str:
        """
        Gets the access token for the gateway.

        Returns:
            str: Access token for the gateway.
        """
        return self.at

    async def sendCommandToDevice(self, deviceId: str, command: Dict[str, Any]) -> Dict[str, Any]:
        """
        Updates specific device information or state.

        Args:
            deviceId (str): ID of the device to send the command to.
            command (Dict[str, Any]): Command to be sent to the device.

        Returns:
            Dict[str, Any]: Dictionary containing the response data.
        """
        return await self.httpRequest(
            path=EPath.DEVICE.value,
            method=EMethod.PUT,
            params={'deviceId': deviceId, 'command': command}
        )

    async def getDebugLog(self) -> Dict[str, Any]:
        """
        Gets the debug log interface.

        Returns:
            Dict[str, Any]: Dictionary containing the response data.
        """
        return await self.httpRequest(
            path=EPath.DEBUG_LOG.value,
            method=EMethod.GET
        )



"""
API Module: nspanelproClass

This module provides the NSPanelProClass API.

Classes:
    NSPanelProClass: Represents the NSPanelProClass API.

Usage:
    from .nspanelproClass import NSPanelProClass
"""

from .baseClass import BaseClass


class NSPanelProClass(BaseClass):
    """
    Represents the NSPanelProClass API.

    Inherits:
        BaseClass: Base class for the API.

    Methods:
        __init__(ip: str, at: str = None, debug: bool = False): Initializes the NSPanelProClass object.
    """

    def __init__(self, ip: str, at: str = None, debug: bool = False):
        """
        Initializes the NSPanelProClass object.

        Parameters:
            ip (str): The IP address of the host.
            at (str): The access token.
            debug (bool): Whether to enable debug mode.

        """
        super().__init__(ip=ip, at=at, debug=debug)



"""
Module: IDebugLog

This module defines the IDebugLog class.

Classes:
    IDebugLog: Represents a debug log with its associated attributes.
"""
#pylint: disable-msg=too-few-public-methods

class IDebugLog:
    """
    Represents a debug log.

    Attributes:
        serial_number (str): The serial number associated with the debug log.
        type (str): The type of the debug log.
        from_index (int): The starting index of the debug log.
        start_time (int): The start time of the debug log.
        end_time (int): The end time of the debug log.
        limit (int): The limit of the debug log.
        order (str): The order of the debug log.
    """

    def __init__(
        self,
        serial_number: str,
        log_type: str,
        from_index: int = None,
        start_time: int = None,
        end_time: int = None,
        limit: int = None,
        order: str = None,
    ):
        self.serial_number = serial_number
        self.type = log_type
        self.from_index = from_index
        self.start_time = start_time
        self.end_time = end_time
        self.limit = limit
        self.order = order



"""
Interface module: ISseEvent

This module defines the ISseEvent class.

Classes:
    ISseEvent: Represents an SSE event object with optional callbacks for various events.
"""
#pylint: disable-msg=too-few-public-methods

from typing import Optional


class ISseEvent:
    """
    Represents an SSE event object.

    Attributes:
        onopen (Optional[callable]): The callback function for the 'onopen' event.
        onerror (Optional[callable]): The callback function for the 'onerror' event.
        onAddDevice (Optional[callable]): The callback function for the 'onAddDevice' event.
        onUpdateDeviceState (Optional[callable]): The callback function for the 'onUpdateDeviceState' event.
        onUpdateDeviceInfo (Optional[callable]): The callback function for the 'onUpdateDeviceInfo' event.
        onUpdateDeviceOnline (Optional[callable]): The callback function for the 'onUpdateDeviceOnline' event.
        onDeleteDevice (Optional[callable]): The callback function for the 'onDeleteDevice' event.
    """

    def __init__(
        self,
        onopen: Optional[callable] = None,
        onerror: Optional[callable] = None,
        onAddDevice: Optional[callable] = None,
        onUpdateDeviceState: Optional[callable] = None,
        onUpdateDeviceInfo: Optional[callable] = None,
        onUpdateDeviceOnline: Optional[callable] = None,
        onDeleteDevice: Optional[callable] = None,
    ):
        self.onopen = onopen
        self.onerror = onerror
        self.onAddDevice = onAddDevice
        self.onUpdateDeviceState = onUpdateDeviceState
        self.onUpdateDeviceInfo = onUpdateDeviceInfo
        self.onUpdateDeviceOnline = onUpdateDeviceOnline
        self.onDeleteDevice = onDeleteDevice



"""
Interface module: IHttpConfig

This module defines the IHttpConfig class.

Classes:
    IHttpConfig: Represents an HTTP configuration with properties.
"""
#pylint: disable-msg=too-few-public-methods

from typing import Optional

from ..enum.EMethod import EMethod


class IHttpConfig:
    """
    Represents an HTTP configuration.

    Attributes:
        path (str): The path of the HTTP request.
        method (EMethod): The HTTP method.
        params (Optional[any]): The optional parameters of the HTTP request.
        isNeedAT (bool): Indicates if authentication token is required.
    """

    def __init__(
        self,
        path: str,
        method: EMethod,
        params: Optional[any] = None,
        isNeedAT: bool = False
    ):
        self.path = path
        self.method = method
        self.params = params
        self.isNeedAT = isNeedAT



"""
Module: baseClass

This module provides the BaseClassBridge API.

Classes:
    BaseClassBridge: Represents the BaseClassBridge API.

Usage:
    from .baseClassBridge import BaseClassBridge
"""

from typing import Any, Awaitable, Dict, Tuple

import asyncio

from ..ts.enum.EMethod import EMethod
from ..ts.enum.EPath import EPath
from ..config import Store
from ..utils.httpUtils import httpUtils


class BaseClassBridge(Store, httpUtils):
    """
    BaseClassBridge API.

    Attributes:
        interval (int): Interval for event subscription.
        timeout (int): Timeout for getting the bridge access token.

    Methods:
        getBridgeAT(timeout: int = 120000, interval: int = 2000) -> Dict[str, Any]: Gets the bridge access token.
        updateBridgeConfig(volume: int) -> Dict[str, Any]: Updates the gateway configuration.
        getBridgeInfo() -> Dict[str, Any]: Gets the gateway information.
    """
    interval = None
    timeout = None

    async def getBridgeAT(self, timeout: int = 120000, interval: int = 2000) -> Dict[str, Any]:
        """
        Gets the bridge access token.

        Args:
            timeout (int): Timeout in milliseconds (default: 120000).
            interval (int): Interval between attempts in milliseconds (default: 2000).

        Returns:
            Dict[str, Any]: Dictionary containing the response data.
        """

        async def getBridgeATHandler() -> Awaitable[Tuple[Dict[str, Any], Dict[str, Any]]]:
            """
            Internal function to handle getting the bridge access token.

            Returns:
                Dict[str, Any]: Dictionary containing the response data.
            """
            resp = await self.httpRequest(
                path=EPath.BRIDGE_TOKEN.value,
                method=EMethod.GET,
                isNeedAT=False
            )

            # Set access token
            if resp['error'] == 0:
                if 'token' in resp['data']:
                    self.setAT(resp['data']['token'])
                return resp

        async def intervalFunc() -> Dict[str, Any]:
            """
            Internal function to handle interval-based attempts to get the bridge access token.
            """
            while True:
                resp = await getBridgeATHandler()
                if resp:
                    return resp
                await asyncio.sleep(interval / 1000)

        return await asyncio.wait_for(asyncio.gather(getBridgeATHandler(), intervalFunc()), timeout / 1000)


    async def updateBridgeConfig(self, volume: int) -> Dict[str, Any]:
        """
        Updates the gateway configuration.

        Args:
            volume (int): System Volume value to be updated. [0-100]

        Returns:
            Dict[str, Any]: Dictionary containing the response data.
        """
        return await self.httpRequest(
            path=EPath.BRIDGE_CONFIG.value,
            method=EMethod.PUT,
            params={'volume': volume}
        )

    async def getBridgeInfo(self) -> Dict[str, Any]:
        """
        Gets the gateway information.

        Returns:
            Dict[str, Any]: Dictionary containing the response data.
        """
        return await self.httpRequest(
           path=EPath.BRIDGE.value,
           method=EMethod.GET,
           isNeedAT=False
        )



"""
Interface module: IConfig

This module defines the IConfig class, which represents a configuration object.

Classes:

    IConfig: Configuration class with properties for IP address, access token, MAC address, and debug mode.
"""
#pylint: disable-msg=too-few-public-methods

import aiohttp


class IConfig:
    """
    Represents a configuration object.

    Attributes:
        ip (str): The IP address associated with the configuration.
        at (str): The access token associated with the configuration.
        mac (str): The MAC address associated with the configuration.
        debug (bool): The debug mode setting of the configuration.
    """

    def __init__(self, ip: str, at: str, mac: str, debug: bool, session: aiohttp.ClientSession):
        self.ip = ip
        self.at = at
        self.mac = mac
        self.debug = debug
        self.session = session



"""
Enumeration: ECategory

This enumeration defines different device categories.

Enumerations:
    ECategory: Represents various device categories.
"""

from enum import Enum


class ECategory(Enum):
    """
    Represents various device categories.

    Enumerations:
        PLUG: Plug category.
        SWITCH: Switch category.
        LIGHT: Light category.
        CURTAIN: Curtain category.
        CONTACT_SENSOR: Contact sensor category.
        MOTION_SENSOR: Motion sensor category.
        TEMPERATURE_SENSOR: Temperature sensor category.
        HUMIDITY_SENSOR: Humidity sensor category.
        TEMPERATURE_HUMIDITY_SENSOR: Temperature and humidity sensor category.
        WATER_LEAK_DETECTOR: Water leak detector category.
        SMOKE_DETECTOR: Smoke detector category.
        BUTTON: Button category.
    """
    PLUG = "plug"
    SWITCH = "switch"
    LIGHT = "light"
    CURTAIN = "curtain"
    CONTACT_SENSOR = "contactSensor"
    MOTION_SENSOR = "motionSensor"
    TEMPERATURE_SENSOR = "temperatureSensor"
    HUMIDITY_SENSOR = "humiditySensor"
    TEMPERATURE_HUMIDITY_SENSOR = "temperatureAndHumiditySensor"
    WATER_LEAK_DETECTOR = "waterLeakDetector"
    SMOKE_DETECTOR = "smokeDetector"
    BUTTON = "button"



"""Setup file for installing and configuring the sonoff-ewelink-cube-client-api package."""
from setuptools import setup, find_packages

VERSION = "0.0.1"

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read().splitlines()

setup(
    name="sonoff-ewelink-cube-client-api",
    version=VERSION,
    author="sipimokus",
    author_email="sipimokus@gmail.com",
    description="SONOFF eWelink CUBE API communication library (unofficial)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/sm4rth0m3/python-pip.sonoff-ewelink-cube-client-api",
    project_urls={
        "Bug Tracker": "https://github.com/sm4rth0m3/python-pip.sonoff-ewelink-cube-client-api/issues",
        "Documentation": "https://github.com/sm4rth0m3/python-pip.sonoff-ewelink-cube-client-api",
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        # Need cross OS and python tests
        # "Programming Language :: Python :: 3.6",
        # "Programming Language :: Python :: 3.7",
        # "Programming Language :: Python :: 3.8",
        # "Programming Language :: Python :: 3.9",
        # "Programming Language :: Python :: 3.10",
        # "Programming Language :: Python :: 3.11",
        "Topic :: Home Automation",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Networking",
        "Topic :: Utilities",
    ],
    keywords="sonoff ewelink cube api openapi ihost nspanel library asyncio",
    packages=find_packages("src"),
    package_dir={"": "src"},
    python_requires=">=3.6",
    install_requires=requirements,
)



"""
Module: config/__init__.py

This module defines the config Store class for managing configurations.
"""

from ..ts.interface.IConfig import IConfig


class Store:
    """
    Class representing a configuration store.

    Attributes:
        config (IConfig): Configuration object.
    """

    ip = None
    at = None
    mac = None

    def __init__(self):
        """
        Initialize a new instance of the Store class.
        """
        self.config: IConfig

    def setConfig(self, debug: bool = None):
        """
        Set the configuration debug flag.

        Args:
            debug (bool): Debug flag value.
        """
        if debug is not None:
            self.config.debug = debug

    def setIp(self, ip: str):
        """
        Set the configuration IP address.

        Args:
            ip (str): IP address value.
        """
        self.config.ip = ip

    def setAT(self, at: str):
        """
        Set the configuration access token.

        Args:
            at (str): Access token value.
        """
        self.config.at = at

    def setMac(self, mac: str):
        """
        Set the configuration MAC address.

        Args:
            mac (str): MAC address value.
        """
        self.config.mac = mac



"""
Interface module: IDevice

This module defines the IDevice class and ICapability class.

Classes:
    ICapability: Represents a capability with its permission and optional name.
    IDevice: Represents a device with its properties.
"""
#pylint: disable-msg=too-few-public-methods
#pylint: disable-msg=too-many-arguments
#pylint: disable-msg=too-many-instance-attributes

from typing import Optional, List

from ..enum.ECapability import ECapability
from ..enum.ECategory import ECategory


class ICapability:
    """
    Represents a capability.

    Attributes:
        capability (ECapability): The capability value.
        permission (str): The permission associated with the capability.
        name (Optional[str]): The optional name of the capability.
    """

    def __init__(
        self,
        capability: ECapability,
        permission: str,
        name: Optional[str] = None
    ):
        self.capability = capability
        self.permission = permission
        self.name = name


class IDevice:
    """
    Represents a device.

    Attributes:
        serial_number (str): The serial number of the device.
        name (str): The name of the device.
        manufacturer (str): The manufacturer of the device.
        model (str): The model of the device.
        firmware_version (str): The firmware version of the device.
        display_category (ECategory): The display category of the device.
        link_layer_type (Optional[str]): The optional link layer type of the device.
        capabilities (List[ICapability]): The list of capabilities associated with the device.
        state (Optional[any]): The optional state of the device.
        online (bool): The online status of the device.
        tags (Optional[any]): The optional tags associated with the device.
    """

    def __init__(
        self,
        serial_number: str,
        name: str,
        manufacturer: str,
        model: str,
        firmware_version: str,
        display_category: ECategory,
        link_layer_type: Optional[str] = None,
        capabilities: List[ICapability] = None,
        state: Optional[any] = None,
        online: bool = None,
        tags: Optional[any] = None
    ):
        self.serial_number = serial_number
        self.name = name
        self.manufacturer = manufacturer
        self.model = model
        self.firmware_version = firmware_version
        self.display_category = display_category
        self.link_layer_type = link_layer_type
        self.capabilities = capabilities or []
        self.state = state
        self.online = online
        self.tags = tags
